##############################################################################
#
#   Module to support witholding tax for OpenERP
#   Copyright (C) 2009, Almacom (Thailand) Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv,fields
import time

class account_tax_code(osv.osv):
    _inherit="account.tax.code"
    _columns={
        "is_wht": fields.boolean("Withholding Tax"),
    }
account_tax_code()

class purchase_order(osv.osv):
    _inherit="purchase.order"

    def _amount_all(self, cr, uid, ids, field_name, arg, context):
        res = {}
        cur_obj=self.pool.get('res.currency')
        for order in self.browse(cr, uid, ids):
            res[order.id] = {
                'amount_untaxed': 0.0,
                'amount_tax': 0.0,
                'amount_total': 0.0,
                'amount_wht': 0.0,
            }
            val = val1 = val_wht = 0.0
            cur=order.pricelist_id.currency_id
            for line in order.order_line:
                taxes_id=[tax for tax in line.taxes_id if not tax.tax_code_id.is_wht]
                for c in self.pool.get('account.tax').compute(cr, uid, taxes_id, line.price_unit, line.product_qty, order.partner_address_id.id, line.product_id, order.partner_id):
                    val+= c['amount']
                wht_taxes_id=[tax for tax in line.taxes_id if tax.tax_code_id.is_wht]
                for c in self.pool.get('account.tax').compute(cr, uid, wht_taxes_id, line.price_unit, line.product_qty, order.partner_address_id.id, line.product_id, order.partner_id):
                    val_wht+= c['amount']
                val1 += line.price_subtotal
            res[order.id]['amount_tax']=cur_obj.round(cr, uid, cur, val)
            res[order.id]['amount_untaxed']=cur_obj.round(cr, uid, cur, val1)
            res[order.id]['amount_total']=res[order.id]['amount_untaxed'] + res[order.id]['amount_tax']
            res[order.id]['amount_wht']=cur_obj.round(cr, uid, cur, val_wht)
        return res

    def _get_order(self, cr, uid, ids, context={}):
        result = {}
        for line in self.pool.get('purchase.order.line').browse(cr, uid, ids, context=context):
            result[line.order_id.id] = True
        return result.keys()

    _columns={
        'amount_untaxed': fields.function(_amount_all, method=True, string='Untaxed Amount',
            store={
                'purchase.order.line': (_get_order, None, 10),
            }, multi="sums"),
        'amount_tax': fields.function(_amount_all, method=True, string='Taxes',
            store={
                'purchase.order.line': (_get_order, None, 10),
            }, multi="sums"),
        'amount_total': fields.function(_amount_all, method=True, string='Total',
            store={
                'purchase.order.line': (_get_order, None, 10),
            }, multi="sums"),
        'amount_wht': fields.function(_amount_all, method=True, string='Withholding Taxes',
            store={
                'purchase.order.line': (_get_order, None, 10),
            }, multi="sums"),
    }
purchase_order()

class account_invoice(osv.osv):
    _inherit="account.invoice"

    def _amount_all(self, cr, uid, ids, name, args, context=None):
        res = {}
        for invoice in self.browse(cr,uid,ids, context=context):
            res[invoice.id] = {
                'amount_untaxed': 0.0,
                'amount_tax': 0.0,
                'amount_total': 0.0,
                'amount_wht': 0.0
            }
            for line in invoice.invoice_line:
                res[invoice.id]['amount_untaxed'] += line.price_subtotal
            for line in invoice.tax_line:
                if line.is_wht:
                    res[invoice.id]['amount_wht'] += line.amount
                else:
                    res[invoice.id]['amount_tax'] += line.amount
            res[invoice.id]['amount_total'] = res[invoice.id]['amount_tax'] + res[invoice.id]['amount_untaxed']
        return res

    def _get_invoice_line(self, cr, uid, ids, context=None):
        result = {}
        for line in self.pool.get('account.invoice.line').browse(cr, uid, ids, context=context):
            result[line.invoice_id.id] = True
        return result.keys()

    def _get_invoice_tax(self, cr, uid, ids, context=None):
        result = {}
        for tax in self.pool.get('account.invoice.tax').browse(cr, uid, ids, context=context):
            result[tax.invoice_id.id] = True
        return result.keys()

    _columns={
        'amount_untaxed': fields.function(_amount_all, method=True, digits=(16,2),string='Untaxed',
            store={
                'account.invoice': (lambda self, cr, uid, ids, c={}: ids, None, 20),
                'account.invoice.tax': (_get_invoice_tax, None, 20),
                'account.invoice.line': (_get_invoice_line, None, 20),
            },
            multi='all'),
        'amount_tax': fields.function(_amount_all, method=True, digits=(16,2), string='Tax',
            store={
                'account.invoice': (lambda self, cr, uid, ids, c={}: ids, None, 20),
                'account.invoice.tax': (_get_invoice_tax, None, 20),
                'account.invoice.line': (_get_invoice_line, None, 20),
            },
            multi='all'),
        'amount_total': fields.function(_amount_all, method=True, digits=(16,2), string='Total',
            store={
                'account.invoice': (lambda self, cr, uid, ids, c={}: ids, None, 20),
                'account.invoice.tax': (_get_invoice_tax, None, 20),
                'account.invoice.line': (_get_invoice_line, None, 20),
            },
            multi='all'),
        'amount_wht': fields.function(_amount_all, method=True, digits=(16,2), string='Withholding Tax',
            store={
                'account.invoice': (lambda self, cr, uid, ids, c={}: ids, None, 20),
                'account.invoice.tax': (_get_invoice_tax, None, 20),
                'account.invoice.line': (_get_invoice_line, None, 20),
            },
            multi='all'),
    }
account_invoice()

class account_invoice_tax(osv.osv):
    _inherit="account.invoice.tax"

    _columns={
        'is_wht': fields.related("tax_code_id","is_wht",type="boolean",string='Withhold'),
    }
account_invoice_tax()

class res_partner(osv.osv):
    _name="res.partner"
    _inherit="res.partner"
    _columns={
        "tin":fields.char("Tax Identification Number",size=64)
    }
res_partner()
